#include "FtpLibrary.h"

static FTP *ftp_table[256];
static syntax();

FILE *FtpFullOpen(char *file, char *mode)
{
  FTP *ftp;
  FILE *tmp;
  String Host, User, Passwd, RemoteFile;

  if ( ! syntax (file, Host, User, Passwd, RemoteFile))
    {
      tmp = fopen(file, mode);
      ftp_table[fileno(tmp)] = NULL;
      return tmp;
    }

  if ( FtpLogin(&ftp, Host, User, Passwd, NULL) < 1 )
    return (FILE *) NULL;

  FtpBinaryM(ftp);

  switch(mode[0])
    {
    case 'r': 
	if (FtpOpenReadM(ftp, RemoteFile) < 1)
	  return (FILE *) NULL;
	ftp_table[fileno(ftp->datafile)] = ftp;
	return ftp->datafile;
    case 'w':
	if (FtpOpenWriteM(ftp, RemoteFile) < 1)
	  return (FILE *) NULL;
	ftp_table[fileno(ftp->datafile)] = ftp;
	return ftp->datafile;
    case 'a':
	if (FtpOpenAppendM(ftp, RemoteFile) < 1)
	  return (FILE *) NULL;
	ftp_table[fileno(ftp->datafile)] = ftp;
	return ftp->datafile;
    }
}

int FtpSFullOpen(char *file, char *mode)
{
  FTP *ftp;
  String Host,User,Passwd,RemoteFile;
  STATUS i;

  if ( ! syntax (file, Host, User, Passwd, RemoteFile))
    return -1;

  if ( (i=FtpLogin(&ftp, Host, User, Passwd, NULL)) < 1 )
    return -1;

  FtpBinary(ftp);

  switch(mode[0])
    {
    case 'r': 
	if (FtpSOpenReadM(ftp, RemoteFile) < 1) 
	  return -1;
	ftp_table[ftp->datasock] = ftp;
	return ftp->datasock;
    case 'w':
	if (FtpSOpenWriteM(ftp, RemoteFile) < 1)
	  return -1;
	ftp_table[ftp->datasock] = ftp;
	return ftp->datasock;
    case 'a':
	if (FtpSOpenAppendM(ftp, RemoteFile) < 1)
	  return -1;
	ftp_table[ftp->datasock] = ftp;
	return ftp->datasock;
    }
}

STATUS FtpFullClose (FILE *f)
{
  FTP *ftp=ftp_table[fileno(f)];
  if (ftp == NULL)
    return fclose(f);
  FtpClose(ftp);
  return FtpBye(ftp);
}

STATUS FtpSFullClose (int s)
{
  FTP *ftp=ftp_table[s];
  if (ftp == NULL) return -1;
  FtpClose(ftp);
  return FtpBye(ftp);
}

static syntax (source, host, user, passwd, file)
String source, host, user, passwd, file;
{
  char *in,*out;
  host[0] = user[0] = passwd[0] = file[0] = '\0';
  for ( in = source , out = host;
       *in !='\0' && *in != '/' ;
       *out++ = *in++);
  *out = '\0';
  if ( *in == '\0' ) return 0;
  in++;
  for ( out = user;
       *in !='\0' && *in != '/' ;
       *out++ = *in++);
  *out = '\0';
  if ( *in == '\0' ) return 0;
  in++;
  for ( out = passwd;
       *in !='\0' && *in != ':' ;
       *out++ = *in++);
  *out = '\0';
  if ( *in == '\0' ) return 0;
  in++;
  for ( out = file;
       *in !='\0';
       *out++ = *in++);
  *out = '\0';
  return 1;
}
